import xml.etree.ElementTree as ElementTree
import urllib.parse
from .BaseClient import BaseClient
from ..parsers.parse_xml_to_json import parse_xml_to_json


class PrestaShopClient(BaseClient):
    def __init__(self, base_url, token):
        self.token = token
        self.base_url = base_url

    def build_url(self, url, params=None):
        updated_params = {**(params or {}), 'ws_key': self.token}
        return f'{self.base_url}/api{url}?{urllib.parse.urlencode(updated_params)}'

    def request_xml(self, url, params=None, timeout=20):
        constructed_params = {'display': 'full', **(params or {})}
        full_url = self.build_url(url, constructed_params)
        response = self.get(full_url, timeout=timeout)
        xml = ElementTree.fromstring(response.content)
        result = [parse_xml_to_json(child) for child in list(xml)[0]]
        return result

    def request(self, url, params=None, timeout=20):
        constructed_params = {
            'display': 'full',
            'output_format': 'JSON',
            'ps_method': 'GET',
            **(params or {})
        }
        full_url = self.build_url(url, constructed_params)
        response = self.get(full_url, timeout=timeout)
        return response.json()

    def get_products(self, params=None):
        return self.normalise(self.request('/products', params=params), 'products')

    def get_categories(self, params=None):
        return self.normalise(self.request('/categories', params), 'categories')

    def get_features(self, params=None):
        return self.normalise(self.request('/product_features', params), 'product_features')

    def get_feature_values(self, params=None):
        return self.normalise(self.request('/product_feature_values', params), 'product_feature_values')

    def get_combinations(self, params=None):
        return self.normalise(self.request('/combinations', params), 'combinations')

    def get_product_options(self, params=None):
        return self.normalise(self.request('/product_options', params), 'product_options')

    def get_product_option_values(self, params=None):
        return self.normalise(self.request('/product_option_values', params), 'product_option_values')

    def get_images(self, params=None):
        return self.normalise(self.request('/images', params), 'images')

    def get_tax_rule_groups(self, params=None):
        return self.normalise(self.request('/tax_rule_groups', params), 'tax_rule_groups')

    def get_tax_rules(self, params=None):
        return self.normalise(self.request('/tax_rules', params), 'tax_rules')

    def get_taxes(self, params=None):
        return self.normalise(self.request('/taxes', params), 'taxes')

    def get_specific_prices(self, params=None):
        return self.normalise(self.request('/specific_prices', params), 'specific_prices')

    @staticmethod
    def normalise(array, key):
        if array is None:
            return []
        elif isinstance(array, dict):
            return array.get(key)
        elif isinstance(array, list):
            return array
        return []
